/*
 *      Mdulo............: DLGFILE.PRG
 *      Utilidad..........: Muestra la caja de dilogo correspondiente a la
 *                          seleccin de un fichero.
 *
 *                          Para probar la funcin directamente desde ste
 *                          fuente debe incluir el comando INIT GRAPHICS.
 *
 *      Fecha de creacin.: 26-02-1996
 *      Ultima revisin...: 07-05-1996
 *
 *      Copyright (c) 1995, 96 por Gabriel Muoz Mrquez
 *      Todos los derechos reservados
 */

#include "ogl.ch"

STATIC cEspec

FUNCTION DlgFile( cTitulo, cLEspec )
   
   LOCAL   cFichero  := "", ;
	   cCurDrive := SetDrive(), ;           // Unidad actual 
	   cCurDir   := "\"+CurDir(), ;         // Directorio actual
	   cOldFont, ;
	   cGet
   LOCAL   aFiles, ;                            // Ruta por defecto
	   aDir      := Dir( "*.*" ), ;         // Todos los directorios
	   aDrives   := ADrives()
   LOCAL   oDialog
   LOCAL   lAceptar  := .F.
   LOCAL   nCombo
   
   DEFAULT cTitulo := "Abrir fichero", ;
	   cLEspec := "*.*"
	   
   cEspec := cLEspec
   cGet   := cEspec
   aFiles := Files( cEspec )    

   // INIT GRAPHICS PATH "C:\CLIPPER5\OGL"

   DEFINE DIALOG oDialog ;
      FROM 60, 100 TO 430, 510 ;
      TITLE cTitulo ;
      EXECUTE ;
	 MSOff(), ;
	 cOldFont := SetFont( OGLDir() + "OGL000.DAT", "TIMES" ), ;
	 GBox( oDialog:nTop + 343, oDialog:nLeft + 5, oDialog:nBottom - 9, oDialog:nRight - 10, 1 ), ;
	 GTextShadow( oDialog:nTop + 344, oDialog:nLeft + 10, SetDrive() + "\" + SubStr( CurDir(), 1, 43 ), ColorToN( "GR+" ), ColorToN( "N" ) ), ;
	 SetFont( OGLDir() + "OGL000.DAT", cOldFont ), ;
	 MSOn()

   @ 20, 325 BUTTON "Aceptar" ;
      EXECUTE lAceptar := .T., oDialog:lEnd := .T. ;
      SIZE 20, 65 ;
      OF oDialog

   @ 50, 325 BUTTON "Cancelar" ;
      EXECUTE lAceptar := .F., oDialog:lEnd := .T. ;
      SIZE 20, 65 ;
      OF oDialog

   @ 260, 180 COMBOBOX nCombo ;
      ITEMS aDrives ;
      HEIGHT 3 ;
      TITLE "Unidad" ;
      DEFAULT AScan( aDrives, SetDrive() ) ;
      BLOCK { | nDrive, cDrive | ChDrive( nDrive, cDrive, oDialog ) } ;
      OF oDialog

   @ 30, 17 GET cGet ;
      TITLE "Nombre" ;
      VALID ChEspec( cGet, oDialog ) ;
      ID 101 ;
      OF oDialog

   @ 60, 190 SAY "Directorios" ;
      OF oDialog
      
   @ 75, 180 LISTBOX aDir ;
      SIZE 8, 125 ;
      BLOCK { | nFile, cFile | ChFiles( nFile, cFile, oDialog, aDrives[ nCombo ] ) } ;
      ID 102 ;
      OF oDialog

   @ 60, 30 SAY "Ficheros" ;
      OF oDialog

   @ 75, 20 LISTBOX aFiles ;
      SIZE 12, 125 ;
      BLOCK { | nFile, cFile | ChGet( nFile, cFile, oDialog ) } ;
      ID 103 ;
      OF oDialog

   ACTIVATE oDialog
   
   if lAceptar .and. ! "*" $ cEspec .and. ! "?" $ cEspec
      cFichero := SetDrive() + "\" + CurDir() + if( Len( CurDir() ) > 0, "\", "" ) + cEspec
      cFichero := if( !File( cFichero ), "", cFichero )
   endif

   SetDrive( cCurDrive )
   ChDir( cCurDir )

return( cFichero )

/* ==== FUNCION ESTATICA Files()
 *      Devuelve en forma de array los ficheros de la unidad y directorio
 *      indicados.
 */
STATIC FUNCTION Files( cRuta )
      
   LOCAL aArbol := Directory( cRuta ), ;
	 aFiles := {}

   AEval( aArbol, { | aArray | AAdd( aFiles, aArray[1] ) } ) 

return( ASort( aFiles ) )

/* ==== FUNCION ESTATICA Dir()
 *      Devuelve en forma de array los directorios de la unidad y 
 *      directorio indicados.
 */
STATIC FUNCTION Dir( cRuta )
   
   LOCAL aArbol := Directory( cRuta, "D" ), ;
	 aDir   := {}

   AEval( aArbol, { | aArray | if( "D" $ aArray[5], AAdd( aDir, aArray[1] ),) } ) 

return( ASort( aDir ) )

/* ==== FUNCION ESTATICA ADrives()
 *      Devuelve en forma de array todas las unidades diponibles
 */
STATIC FUNCTION ADrives()

   LOCAL aDrives := { "A:" + Space( 11 ), "B:" + Space( 11 ) }  // Siempre habr A y B
   LOCAL nCont                                                  // Variable contador

   // Aadimos a aDrives los discos duros
   for nCont := 1 to HardDisks()
      AAdd( aDrives, Chr( 65 + nCont + 1 ) + ":" + Volumen( Chr( 65 + nCont + 1 ) ) )
   next

   // Aadimos las disqueteras
   for nCont := HardDisks()+2 to HardDisks()+PhyDisks()-1
      AAdd( aDrives, Chr( 65 + nCont ) + ":" ) 
   next

   // Aadimos el resto de unidades  
   for nCont := Len( aDrives ) to LogDrives() - 1
      AAdd( aDrives, Chr( 65 + nCont ) + ":" + Volumen( Chr( 65 + nCont ) ) )
   next

return( aDrives )

/* ==== FUNCION ESTATICA ChFiles()
 *      Cambia en tiempo de ejecucin los ficheros y directorios
 */
STATIC FUNCTION ChFiles( nFile, cFile, oDialog, cDrive )

   LOCAL aFiles, ;
	 aDir
   LOCAL cOldFont

   if GetDrive( cDrive ) != D_NOPREPARADA   
      SetDrive( cDrive )              // Cambiamos de unidad 
      ChDir( cFile )                  // Cambiamos al directorio indicado
      aFiles := Files( cEspec )       // Slo ficheros con la especificacin
      aDir   := Dir( "*.*" )          // Todos los directorios

      REDEFINE oDialog GET     ID 101 WITH cEspec           
      REDEFINE oDialog LISTBOX ID 102 WITH aDir
      REDEFINE oDialog LISTBOX ID 103 WITH aFiles

   else
      Error()
   endif

   MSOff()
   cOldFont := SetFont( OGLDir() + "OGL000.DAT", "TIMES" )
   GBox( oDialog:nTop + 343, oDialog:nLeft + 5, oDialog:nBottom - 9, oDialog:nRight - 10, 1 )
   GTextShadow( oDialog:nTop + 344, oDialog:nLeft + 10, SetDrive() + "\" + SubStr( CurDir(), 1, 43 ), ColorToN( "GR+" ), ColorToN( "N" ) )
   SetFont( OGLDir() + "OGL000.DAT", cOldFont )
   MSOn()
 
return( nil )

/* ==== FUNCION ESTATICA ChDrive()
 *      Cambia en tiempo de ejecucin la unidad, ficheros y directorios
 */
STATIC FUNCTION ChDrive( nDrive, cDrive, oDialog )

   if SubStr( cDrive, 1, 1 ) != SetDrive() 
      if GetDrive( cDrive ) != D_NOPREPARADA   
	 ChFiles( 0, ".", oDialog, cDrive, cEspec )
      else
	 Error()
      endif
   endif

return( nil ) 

/* ==== FUNCION ESTATICA ChGet()
 *      Cambia en tiempo de ejecucin el valor que se edita en el objeto
 *      GET
 */
STATIC FUNCTION ChGet( nFile, cFile, oDialog )

   REDEFINE oDialog GET ID 101 WITH cFile
   cEspec := cFile

return( nil )

/* ==== FUNCION ESTATICA ChEspec()
 *      Cambia en tiempo de ejecucin la especificacin y los ficheros
 */
STATIC FUNCTION ChEspec( cGet, oDialog )
	
   LOCAL nPosF, ;
	 nPos
   LOCAL cDrive, ;
	 cDir      := "."

   // Slo cambiar de ficheros cuando la especificacin sea distinta
   // a la actual
   if cEspec != cGet                   
   
      if ( nPosF := Rat( ":", cGet ) ) > 0    // Se detecta si se incluyen directorios
	 cDrive := SubStr( cGet, 1, nPosF )
      endif
      
      if File( cGet )                         // Si existe el fichero
	 nPos  := Rat( "\", cGet )
	 if nPos == nPosF + 1                 // Controlar directorio raz
	    nPosF := nPos
	 else
	    nPosF := if( nPos == 0, nPosF, nPos - 1 )
	 endif                                    
	 cDir := SubStr( cGet, 1, nPosF )
	 cEspec := SubStr( cGet, if( ( nPos := Rat( "\", cGet ) ) > 0, nPos + 1, ;
	      if( ( nPos := Rat( ":", cGet ) ) > 0, nPos + 1, 1 ) ) )
	 ChFiles( 0, cDir, oDialog, cDrive )
      else
	 if GetDrive( cDrive ) == D_NOPREPARADA    
	    Error()
	 endif
	 REDEFINE oDialog GET ID 101 WITH cEspec   // Se vuelve a dejar como estaba
      endif
      
   endif                     

return( .T. )   // Devolver verdadero porque es el VALID del Get.

/* ==== FUNCION ESTATICA Error()
 *      Muestra la caja de dilogo de error
 */
STATIC FUNCTION Error()
   
   LOCAL oDialog
   LOCAL cOldFont

   DEFINE DIALOG oDialog ;
      FROM 155, 185 TO 280, 410 ;
      EXECUTE ;
	 MSOff(), ;
	 cOldFont := SetFont( OGLDir() + "OGL000.DAT", "HELV" ), ;
	 ShowPCX( oDialog:nTop + 40, oDialog:nLeft / 8 + 2, OGLDir() + "OGL000.DAT", "STOP.PCX", .f. ), ;
	 GText( oDialog:nTop + 50, oDialog:nLeft + 70, "Unidad no preparada", 4 ), ;
	 SetFont( OGLDir() + "OGL000.DAT", cOldFont ), ;
	 MSOn() ;
      TITLE "STOP" 
      
   @ 67, 80 BUTTON "Aceptar" ;
      SIZE 20, 65 ;
      EXECUTE oDialog:lEnd := .T. ;
      OF oDialog   
	 
return( ACTIVATE oDialog )   
