/*
 *      Mdulo............: ERRSYSG.PRG
 *      Utilidad..........: Standard Clipper error handler
 *
 *      Adaptado a la librera Object Graph Library.
 *
 *      Copyright (c) 1990, 93, Computer Associates International, Inc.
 *      Todos los derechos reservados
 */

#include "error.ch"
#include "ogl.ch"

#define CRLF Chr( 13 ) + Chr( 10 )

// put messages to STDERR
#command ? <list,...>   =>  ?? Chr(13) + Chr(10) ; ?? <list>
#command ?? <list,...>  =>  OutErr(<list>)


// used below
#define NTRIM(n)                ( LTrim(Str(n)) )

static  lRecursivo := .F.

/***
*       ErrorSys()
*
*       Note:  automatically executes at startup
*/

proc ErrorSys()
	ErrorBlock( {|e| DefError(e)} )
return




/***
*       DefError()
*/
static func DefError(e)
local i, cMessage, aOptions, nChoice



	// by default, division by zero yields zero
	if ( e:genCode == EG_ZERODIV )
		return (0)
	end


	// for network open error, set NETERR() and subsystem default
	if ( e:genCode == EG_OPEN .and. e:osCode == 32 .and. e:canDefault )

		NetErr(.t.)
		return (.f.)                                                                    // NOTE

	end


	// for lock error during APPEND BLANK, set NETERR() and subsystem default
	if ( e:genCode == EG_APPENDLOCK .and. e:canDefault )

		NetErr(.t.)
		return (.f.)                                                                    // NOTE

	end



	// build error message
	cMessage := ErrorMessage(e)

	ShowDialog( cMessage, e )

	QUIT

return (.f.)

/***
*       ErrorMessage()
*/
static func ErrorMessage(e)
local cMessage


	// start error message
	cMessage := if( e:severity > ES_WARNING, "Error ", "Warning " )


	// add subsystem name if available
	if ( ValType(e:subsystem) == "C" )
		cMessage += e:subsystem()
	else
		cMessage += "???"
	end


	// add subsystem's error code if available
	if ( ValType(e:subCode) == "N" )
		cMessage += ("/" + NTRIM(e:subCode))
	else
		cMessage += "/???"
	end


	// add error description if available
	if ( ValType(e:description) == "C" )
		cMessage += ("  " + e:description)
	end


	// add either filename or operation
	if ( !Empty(e:filename) )
		cMessage += (": " + e:filename)

	elseif ( !Empty(e:operation) )
		cMessage += (": " + e:operation)

	end


return (cMessage)

/* ==== FUNCION ESTATICA ShowDialog()
 *      Muestra el dilogo de error
 */
STATIC FUNCTION ShowDialog( cMessage, e )

   LOCAL aArray := {}
   LOCAL i, oDialog
   LOCAL nLeft := 100, nRight := 400

   if !lRecursivo                       // Evitar errores recursivos
      lRecursivo := .T.

      INIT GRAPHICS NOWARNINGS

      if Len( cMessage ) > 30
	 nLeft  := Int( ( 640 - ( Len( cMessage ) * 8 + 50 ) ) / 2 )
	 nRight := Int( nLeft + Len( cMessage ) * 8 + 50 )
      endif

      if ( !Empty(e:osCode) )
	 cMessage += " (DOS Error " + NTRIM(e:osCode) + ") "
      end

      i := 3
      while ( !Empty(ProcName(i)) )
	 AAdd( aArray, "Llamado desde " + Trim(ProcName(i)) + ;
	     "(" + NTRIM(ProcLine(i)) + ")  " )
	  i++
      end
      
      DEFINE DIALOG oDialog FROM 100, nLeft TO 400, nRight TITLE "ERROR FATAL" ;
	 EXECUTE ;
	    MSOff(), ;
	    ShowPCX( oDialog:nTop + 40, oDialog:nLeft / 8 + 2, OGLDir() + "OGL000.DAT", "STOP", .F. ), ;
	    MSOn()
 
      @ 17, 65 SAY cMessage COLOR "R/W" OF oDialog
      
      @ 45, 60 LISTBOX aArray OF oDialog SIZE 10, nRight - nLeft - 95
      
      @ 242, ( nRight - nLeft ) / 2 - 35 BUTTON "Terminar" ;
	 EXECUTE oDialog:lEnd := .T. ;
	 SIZE 20, 65 ;
	 OF oDialog
      
      Tone( 400, 5 )
      ACTIVATE oDialog

      MsOff()

      CLOSE GRAPHICS NOFADE
      lRecursivo := .F.
   endif
   
   // give up
   ErrorLevel(1)
   Cls
   
   ? cMessage
   AEval( aArray, { | cItem | QOut( cItem ) } )
   ?
   
return( nil )
